@@ -13,9 +13,9 @@
  * @category   pear
  * @package    PEAR
  * @author     Greg Beaver <cellog@php.net>
- * @copyright  1997-2006 The PHP Group
+ * @copyright  1997-2008 The PHP Group
  * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
- * @version    CVS: $Id: 10.php,v 1.43 2006/11/01 05:09:05 cellog Exp $
+ * @version    CVS: $Id: 10.php,v 1.53 2008/04/11 01:16:40 dufuz Exp $
  * @link       http://pear.php.net/package/PEAR
  * @since      File available since Release 1.4.0a12
  */
@@ -31,9 +31,9 @@ require_once 'PEAR/REST.php';
  * @category   pear
  * @package    PEAR
  * @author     Greg Beaver <cellog@php.net>
- * @copyright  1997-2006 The PHP Group
+ * @copyright  1997-2008 The PHP Group
  * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
- * @version    Release: 1.5.0
+ * @version    Release: 1.7.2
  * @link       http://pear.php.net/package/PEAR
  * @since      Class available since Release 1.4.0a12
  */
@@ -65,7 +65,7 @@ class PEAR_REST_10
      * @param bool $installed the installed version of this package to compare against
      * @return array|false|PEAR_Error see {@link _returnDownloadURL()}
      */
-    function getDownloadURL($base, $packageinfo, $prefstate, $installed)
+    function getDownloadURL($base, $packageinfo, $prefstate, $installed, $channel = false)
     {
         $channel = $packageinfo['channel'];
         $package = $packageinfo['package'];
@@ -73,14 +73,9 @@ class PEAR_REST_10
         if (!$states) {
             return PEAR::raiseError('"' . $prefstate . '" is not a valid state');
         }
-        $state = $version = null;
-        if (isset($packageinfo['state'])) {
-            $state = $packageinfo['state'];
-        }
-        if (isset($packageinfo['version'])) {
-            $version = $packageinfo['version'];
-        }
-        $info = $this->_rest->retrieveData($base . 'r/' . strtolower($package) . '/allreleases.xml');
+        $state   = isset($packageinfo['state'])   ? $packageinfo['state']   : null;
+        $version = isset($packageinfo['version']) ? $packageinfo['version'] : null;
+        $info = $this->_rest->retrieveData($base . 'r/' . strtolower($package) . '/allreleases.xml', false, false, $channel);
         if (PEAR::isError($info)) {
             return PEAR::raiseError('No releases available for package "' .
                 $channel . '/' . $package . '"');
@@ -88,8 +83,7 @@ class PEAR_REST_10
         if (!isset($info['r'])) {
             return false;
         }
-        $found = false;
-        $release = false;
+        $release = $found = false;
         if (!is_array($info['r']) || !isset($info['r'][0])) {
             $info['r'] = array($info['r']);
         }
@@ -122,11 +116,11 @@ class PEAR_REST_10
                 }
             }
         }
-        return $this->_returnDownloadURL($base, $package, $release, $info, $found);
+        return $this->_returnDownloadURL($base, $package, $release, $info, $found, false, $channel);
     }
 
     function getDepDownloadURL($base, $xsdversion, $dependency, $deppackage,
-                               $prefstate = 'stable', $installed = false)
+                               $prefstate = 'stable', $installed = false, $channel = false)
     {
         $channel = $dependency['channel'];
         $package = $dependency['name'];
@@ -134,14 +128,9 @@ class PEAR_REST_10
         if (!$states) {
             return PEAR::raiseError('"' . $prefstate . '" is not a valid state');
         }
-        $state = $version = null;
-        if (isset($packageinfo['state'])) {
-            $state = $packageinfo['state'];
-        }
-        if (isset($packageinfo['version'])) {
-            $version = $packageinfo['version'];
-        }
-        $info = $this->_rest->retrieveData($base . 'r/' . strtolower($package) . '/allreleases.xml');
+        $state   = isset($dependency['state'])   ? $dependency['state']   : null;
+        $version = isset($dependency['version']) ? $dependency['version'] : null;
+        $info = $this->_rest->retrieveData($base . 'r/' . strtolower($package) . '/allreleases.xml', false, false, $channel);
         if (PEAR::isError($info)) {
             return PEAR::raiseError('Package "' . $deppackage['channel'] . '/' . $deppackage['package']
                 . '" dependency "' . $channel . '/' . $package . '" has no releases');
@@ -152,8 +141,6 @@ class PEAR_REST_10
         $exclude = array();
         $min = $max = $recommended = false;
         if ($xsdversion == '1.0') {
-            $pinfo['package'] = $dependency['name'];
-            $pinfo['channel'] = 'pear.php.net'; // this is always true - don't change this
             switch ($dependency['rel']) {
                 case 'ge' :
                     $min = $dependency['version'];
@@ -177,7 +164,6 @@ class PEAR_REST_10
                 break;
             }
         } else {
-            $pinfo['package'] = $dependency['name'];
             $min = isset($dependency['min']) ? $dependency['min'] : false;
             $max = isset($dependency['max']) ? $dependency['max'] : false;
             $recommended = isset($dependency['recommended']) ?
@@ -188,8 +174,7 @@ class PEAR_REST_10
                 }
             }
         }
-        $found = false;
-        $release = false;
+        $release = $found = false;
         if (!is_array($info['r']) || !isset($info['r'][0])) {
             $info['r'] = array($info['r']);
         }
@@ -230,7 +215,7 @@ class PEAR_REST_10
                     if (!in_array($release['s'], $states)) {
                         // the stability is too low, but we must return the
                         // recommended version if possible
-                        return $this->_returnDownloadURL($base, $package, $release, $info, true);
+                        return $this->_returnDownloadURL($base, $package, $release, $info, true, false, $channel);
                     }
                 }
             }
@@ -248,7 +233,7 @@ class PEAR_REST_10
                 break;
             }
         }
-        return $this->_returnDownloadURL($base, $package, $release, $info, $found);
+        return $this->_returnDownloadURL($base, $package, $release, $info, $found, false, $channel);
     }
 
     /**
@@ -259,30 +244,31 @@ class PEAR_REST_10
      * @param array $release an array of format array('v' => version, 's' => state)
      *                       describing the release to download
      * @param array $info list of all releases as defined by allreleases.xml
-     * @param bool $found determines whether the release was found or this is the next
-     *                    best alternative
+     * @param bool|null $found determines whether the release was found or this is the next
+     *                    best alternative.  If null, then versions were skipped because
+     *                    of PHP dependency
      * @return array|PEAR_Error
      * @access private
      */
-    function _returnDownloadURL($base, $package, $release, $info, $found)
+    function _returnDownloadURL($base, $package, $release, $info, $found, $phpversion = false, $channel = false)
     {
         if (!$found) {
             $release = $info['r'][0];
         }
-        $pinfo = $this->_rest->retrieveCacheFirst($base . 'p/' . strtolower($package) . '/' . 
-            'info.xml');
+        $pinfo = $this->_rest->retrieveCacheFirst($base . 'p/' . strtolower($package) . '/' .
+            'info.xml', false, false, $channel);
         if (PEAR::isError($pinfo)) {
             return PEAR::raiseError('Package "' . $package .
                 '" does not have REST info xml available');
         }
-        $releaseinfo = $this->_rest->retrieveCacheFirst($base . 'r/' . strtolower($package) . '/' . 
-            $release['v'] . '.xml');
+        $releaseinfo = $this->_rest->retrieveCacheFirst($base . 'r/' . strtolower($package) . '/' .
+            $release['v'] . '.xml', false, false, $channel);
         if (PEAR::isError($releaseinfo)) {
             return PEAR::raiseError('Package "' . $package . '" Version "' . $release['v'] .
                 '" does not have REST xml available');
         }
         $packagexml = $this->_rest->retrieveCacheFirst($base . 'r/' . strtolower($package) . '/' .
-            'deps.' . $release['v'] . '.txt', false, true);
+            'deps.' . $release['v'] . '.txt', false, true, $channel);
         if (PEAR::isError($packagexml)) {
             return PEAR::raiseError('Package "' . $package . '" Version "' . $release['v'] .
                 '" does not have REST dependency information available');
@@ -292,7 +278,7 @@ class PEAR_REST_10
             $packagexml = array();
         }
         $allinfo = $this->_rest->retrieveData($base . 'r/' . strtolower($package) .
-            '/allreleases.xml');
+            '/allreleases.xml', false, false, $channel);
         if (!is_array($allinfo['r']) || !isset($allinfo['r'][0])) {
             $allinfo['r'] = array($allinfo['r']);
         }
@@ -325,13 +311,18 @@ class PEAR_REST_10
             break;
         }
         if (isset($pinfo['dc']) && isset($pinfo['dp'])) {
-            $deprecated = array('channel' => (string) $pinfo['dc'],
-                                'package' => trim($pinfo['dp']['_content']));
+            if (is_array($pinfo['dp'])) {
+                $deprecated = array('channel' => (string) $pinfo['dc'],
+                                    'package' => trim($pinfo['dp']['_content']));
+            } else {
+                $deprecated = array('channel' => (string) $pinfo['dc'],
+                                    'package' => trim($pinfo['dp']));
+            }
         } else {
             $deprecated = false;
         }
         if ($found) {
-            return 
+            return
                 array('version' => $releaseinfo['v'],
                       'info' => $packagexml,
                       'package' => $releaseinfo['p']['_content'],
@@ -348,13 +339,14 @@ class PEAR_REST_10
                       'info' => $packagexml,
                       'compatible' => $compatible,
                       'deprecated' => $deprecated,
+                      'php' => $phpversion
                 );
         }
     }
 
-    function listPackages($base)
+    function listPackages($base, $channel = false)
     {
-        $packagelist = $this->_rest->retrieveData($base . 'p/packages.xml');
+        $packagelist = $this->_rest->retrieveData($base . 'p/packages.xml', false, false, $channel);
         if (PEAR::isError($packagelist)) {
             return $packagelist;
         }
@@ -367,9 +359,102 @@ class PEAR_REST_10
         return $packagelist['p'];
     }
 
-    function listAll($base, $dostable, $basic = true, $searchpackage = false, $searchsummary = false)
+    /**
+     * List all categories of a REST server
+     *
+     * @param string $base base URL of the server
+     * @return array of categorynames
+     */
+    function listCategories($base, $channel = false)
     {
-        $packagelist = $this->_rest->retrieveData($base . 'p/packages.xml');
+        $categories = array();
+
+        // c/categories.xml does not exist;
+        // check for every package its category manually
+        // This is SLOOOWWWW : ///
+        $packagelist = $this->_rest->retrieveData($base . 'p/packages.xml', false, false, $channel);
+        if (PEAR::isError($packagelist)) {
+            return $packagelist;
+        }
+        if (!is_array($packagelist) || !isset($packagelist['p'])) {
+            $ret = array();
+            return $ret;
+        }
+        if (!is_array($packagelist['p'])) {
+            $packagelist['p'] = array($packagelist['p']);
+        }
+
+        PEAR::pushErrorHandling(PEAR_ERROR_RETURN);
+        foreach ($packagelist['p'] as $package) {
+                $inf = $this->_rest->retrieveData($base . 'p/' . strtolower($package) . '/info.xml', false, false, $channel);
+                if (PEAR::isError($inf)) {
+                    PEAR::popErrorHandling();
+                    return $inf;
+                }
+                $cat = $inf['ca']['_content'];
+                if (!isset($categories[$cat])) {
+                    $categories[$cat] = $inf['ca'];
+                }
+        }
+        return array_values($categories);
+    }
+
+    /**
+     * List a category of a REST server
+     *
+     * @param string $base base URL of the server
+     * @param string $category name of the category
+     * @param boolean $info also download full package info
+     * @return array of packagenames
+     */
+    function listCategory($base, $category, $info = false, $channel = false)
+    {
+        // gives '404 Not Found' error when category doesn't exist
+        $packagelist = $this->_rest->retrieveData($base.'c/'.urlencode($category).'/packages.xml', false, false, $channel);
+        if (PEAR::isError($packagelist)) {
+            return $packagelist;
+        }
+        if (!is_array($packagelist) || !isset($packagelist['p'])) {
+            return array();
+        }
+        if (!is_array($packagelist['p']) ||
+            !isset($packagelist['p'][0])) { // only 1 pkg
+            $packagelist = array($packagelist['p']);
+        } else {
+            $packagelist = $packagelist['p'];
+        }
+
+        if ($info == true) {
+            // get individual package info
+            PEAR::pushErrorHandling(PEAR_ERROR_RETURN);
+            foreach ($packagelist as $i => $packageitem) {
+                $url = sprintf('%s'.'r/%s/latest.txt',
+                        $base,
+                        strtolower($packageitem['_content']));
+                $version = $this->_rest->retrieveData($url, false, false, $channel);
+                if (PEAR::isError($version)) {
+                    break; // skipit
+                }
+                $url = sprintf('%s'.'r/%s/%s.xml',
+                        $base,
+                        strtolower($packageitem['_content']),
+                        $version);
+                $info = $this->_rest->retrieveData($url, false, false, $channel);
+                if (PEAR::isError($info)) {
+                    break; // skipit
+                }
+                $packagelist[$i]['info'] = $info;
+            }
+            PEAR::popErrorHandling();
+        }
+
+        return $packagelist;
+    }
+
+
+    function listAll($base, $dostable, $basic = true, $searchpackage = false, $searchsummary = false, $channel = false)
+    {
+        $packagelist = $this->_rest->retrieveData($base . 'p/packages.xml', false, false, $channel);
         if (PEAR::isError($packagelist)) {
             return $packagelist;
         }
@@ -384,6 +469,17 @@ class PEAR_REST_10
         if (!is_array($packagelist['p'])) {
             $packagelist['p'] = array($packagelist['p']);
         }
+
+        // only search-packagename = quicksearch !
+        if ($searchpackage && (!$searchsummary || empty($searchpackage))) {
+            $newpackagelist = array();
+            foreach ($packagelist['p'] as $package) {
+                if (!empty($searchpackage) && stristr($package, $searchpackage) !== false) {
+                    $newpackagelist[] = $package;
+                }
+            }
+            $packagelist['p'] = $newpackagelist;
+        }
         PEAR::pushErrorHandling(PEAR_ERROR_RETURN);
         $next = .1;
         foreach ($packagelist['p'] as $progress => $package) {
@@ -400,17 +496,17 @@ class PEAR_REST_10
             if ($basic) { // remote-list command
                 if ($dostable) {
                     $latest = $this->_rest->retrieveData($base . 'r/' . strtolower($package) .
-                        '/stable.txt');
+                        '/stable.txt', false, false, $channel);
                 } else {
                     $latest = $this->_rest->retrieveData($base . 'r/' . strtolower($package) .
-                        '/latest.txt');
+                        '/latest.txt', false, false, $channel);
                 }
                 if (PEAR::isError($latest)) {
                     $latest = false;
                 }
                 $info = array('stable' => $latest);
             } else { // list-all command
-                $inf = $this->_rest->retrieveData($base . 'p/' . strtolower($package) . '/info.xml');
+                $inf = $this->_rest->retrieveData($base . 'p/' . strtolower($package) . '/info.xml', false, false, $channel);
                 if (PEAR::isError($inf)) {
                     PEAR::popErrorHandling();
                     return $inf;
@@ -425,7 +521,7 @@ class PEAR_REST_10
                     };
                 }
                 $releases = $this->_rest->retrieveData($base . 'r/' . strtolower($package) .
-                    '/allreleases.xml');
+                    '/allreleases.xml', false, false, $channel);
                 if (PEAR::isError($releases)) {
                     continue;
                 }
@@ -479,7 +575,7 @@ class PEAR_REST_10
                 }
                 if ($latest) {
                     $d = $this->_rest->retrieveCacheFirst($base . 'r/' . strtolower($package) . '/deps.' .
-                        $latest . '.txt');
+                        $latest . '.txt', false, false, $channel);
                     if (!PEAR::isError($d)) {
                         $d = unserialize($d);
                         if ($d) {
@@ -523,9 +619,9 @@ class PEAR_REST_10
         return $ret;
     }
 
-    function listLatestUpgrades($base, $state, $installed, $channel, &$reg)
+    function listLatestUpgrades($base, $pref_state, $installed, $channel, &$reg)
     {
-        $packagelist = $this->_rest->retrieveData($base . 'p/packages.xml');
+        $packagelist = $this->_rest->retrieveData($base . 'p/packages.xml', false, false, $channel);
         if (PEAR::isError($packagelist)) {
             return $packagelist;
         }
@@ -536,17 +632,15 @@ class PEAR_REST_10
         if (!is_array($packagelist['p'])) {
             $packagelist['p'] = array($packagelist['p']);
         }
-        if ($state) {
-            $states = $this->betterStates($state, true);
-        }
         foreach ($packagelist['p'] as $package) {
             if (!isset($installed[strtolower($package)])) {
                 continue;
             }
             $inst_version = $reg->packageInfo($package, 'version', $channel);
+            $inst_state = $reg->packageInfo($package, 'release_state', $channel);
             PEAR::pushErrorHandling(PEAR_ERROR_RETURN);
             $info = $this->_rest->retrieveData($base . 'r/' . strtolower($package) .
-                '/allreleases.xml');
+                '/allreleases.xml', false, false, $channel);
             PEAR::popErrorHandling();
             if (PEAR::isError($info)) {
                 continue; // no remote releases
@@ -559,25 +653,39 @@ class PEAR_REST_10
             if (!is_array($info['r']) || !isset($info['r'][0])) {
                 $info['r'] = array($info['r']);
             }
+            // $info['r'] is sorted by version number
             foreach ($info['r'] as $release) {
                 if ($inst_version && version_compare($release['v'], $inst_version, '<=')) {
-                    continue;
+                    // not newer than the one installed
+                    break;
                 }
-                if ($state) {
-                    if (in_array($release['s'], $states)) {
-                        $found = true;
-                        break;
-                    }
-                } else {
+
+                // new version > installed version
+                if (!$pref_state) {
+                    // every state is a good state
                     $found = true;
                     break;
+                } else {
+                    $new_state = $release['s'];
+                    // if new state >= installed state: go
+                    if (in_array($new_state, $this->betterStates($inst_state, true))) {
+                        $found = true;
+                        break;
+                    } else {
+                        // only allow to lower the state of package,
+                        // if new state >= preferred state: go
+                        if (in_array($new_state, $this->betterStates($pref_state, true))) {
+                            $found = true;
+                            break;
+                        }
+                    }
                 }
             }
             if (!$found) {
                 continue;
             }
-            $relinfo = $this->_rest->retrieveCacheFirst($base . 'r/' . strtolower($package) . '/' . 
-                $release['v'] . '.xml');
+            $relinfo = $this->_rest->retrieveCacheFirst($base . 'r/' . strtolower($package) . '/' .
+                $release['v'] . '.xml', false, false, $channel);
             if (PEAR::isError($relinfo)) {
                 return $relinfo;
             }
@@ -590,10 +698,10 @@ class PEAR_REST_10
         return $ret;
     }
 
-    function packageInfo($base, $package)
+    function packageInfo($base, $package, $channel = false)
     {
         PEAR::pushErrorHandling(PEAR_ERROR_RETURN);
-        $pinfo = $this->_rest->retrieveData($base . 'p/' . strtolower($package) . '/info.xml');
+        $pinfo = $this->_rest->retrieveData($base . 'p/' . strtolower($package) . '/info.xml', false, false, $channel);
         if (PEAR::isError($pinfo)) {
             PEAR::popErrorHandling();
             return PEAR::raiseError('Unknown package: "' . $package . '" (Debug: ' .
@@ -601,7 +709,7 @@ class PEAR_REST_10
         }
         $releases = array();
         $allreleases = $this->_rest->retrieveData($base . 'r/' . strtolower($package) .
-            '/allreleases.xml');
+            '/allreleases.xml', false, false, $channel);
         if (!PEAR::isError($allreleases)) {
             if (!class_exists('PEAR_PackageFile_v2')) {
                 require_once 'PEAR/PackageFile/v2.php';
@@ -612,7 +720,7 @@ class PEAR_REST_10
             $pf = new PEAR_PackageFile_v2;
             foreach ($allreleases['r'] as $release) {
                 $ds = $this->_rest->retrieveCacheFirst($base . 'r/' . strtolower($package) . '/deps.' .
-                    $release['v'] . '.txt');
+                    $release['v'] . '.txt', false, false, $channel);
                 if (PEAR::isError($ds)) {
                     continue;
                 }
@@ -622,7 +730,7 @@ class PEAR_REST_10
                 $pf->setDeps(unserialize($ds));
                 $ds = $pf->getDeps();
                 $info = $this->_rest->retrieveCacheFirst($base . 'r/' . strtolower($package)
-                    . '/' . $release['v'] . '.xml');
+                    . '/' . $release['v'] . '.xml', false, false, $channel);
                 if (PEAR::isError($info)) {
                     continue;
                 }
@@ -642,8 +750,13 @@ class PEAR_REST_10
         }
         PEAR::popErrorHandling();
         if (isset($pinfo['dc']) && isset($pinfo['dp'])) {
-            $deprecated = array('channel' => (string) $pinfo['dc'],
-                                'package' => trim($pinfo['dp']['_content']));
+            if (is_array($pinfo['dp'])) {
+                $deprecated = array('channel' => (string) $pinfo['dc'],
+                                    'package' => trim($pinfo['dp']['_content']));
+            } else {
+                $deprecated = array('channel' => (string) $pinfo['dc'],
+                                    'package' => trim($pinfo['dp']));
+            }
         } else {
             $deprecated = false;
         }
